/**
 * Copyright (c) 2015 - 2017, Nordic Semiconductor ASA
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form, except as embedded into a Nordic
 *    Semiconductor ASA integrated circuit in a product or a software update for
 *    such product, must reproduce the above copyright notice, this list of
 *    conditions and the following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 * 
 * 3. Neither the name of Nordic Semiconductor ASA nor the names of its
 *    contributors may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 * 
 * 4. This software, with or without modification, must only be used with a
 *    Nordic Semiconductor ASA integrated circuit.
 * 
 * 5. Any software provided in binary form under this license must not be reverse
 *    engineered, decompiled, modified and/or disassembled.
 * 
 * THIS SOFTWARE IS PROVIDED BY NORDIC SEMICONDUCTOR ASA "AS IS" AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL NORDIC SEMICONDUCTOR ASA OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */
/** @file
 * @defgroup nrf_twi_master_example main.c
 * @{
 * @ingroup nrf_twi_example
 * @brief TWI Example Application main file.
 *
 * This file contains the source code for a sample application using TWI.
 */

#include <stdio.h>
#include "boards.h"
#include "app_util_platform.h"
#include "app_timer.h"
#include "nrf_pwr_mgmt.h"
#include "nrf_drv_clock.h"
#include "bsp.h"
#include "app_error.h"
#include "app_timer.h"
#include "app_twi.h"
#include "lm75b.h"
#include "mma7660.h"
#include "compiler_abstraction.h"
#define NRF_LOG_MODULE_NAME "APP"
#include "nrf_log.h"
#include "nrf_log_ctrl.h"
#include "nrf_delay.h"

#define TWI_INSTANCE_ID             0
#define MAX_PENDING_TRANSACTIONS    5

#define TWI_SLAVE_ADDR              0x50
#define TWI_SLAVE_REG_LEN           7

APP_TWI_DEF(m_app_twi, MAX_PENDING_TRANSACTIONS, TWI_INSTANCE_ID);

static ret_code_t twi_init(void)
{
	ret_code_t err_code;
	nrf_drv_twi_config_t twi_config=NRF_DRV_TWI_DEFAULT_CONFIG;

	twi_config.sda=26;
	twi_config.scl=2;

	err_code=app_twi_init(&m_app_twi, &twi_config);

	APP_ERROR_CHECK(err_code);

	return err_code;
}

static void read_reg(const uint8_t reg_addr, uint8_t* const p_buff, const uint8_t len)
{
    ret_code_t err_code;
	
	static uint8_t tx_buff[1];
	tx_buff[0]=reg_addr;
	app_twi_transfer_t const twi_transfers[] = {
		APP_TWI_WRITE(TWI_SLAVE_ADDR, tx_buff, sizeof(tx_buff), APP_TWI_NO_STOP),
		APP_TWI_READ(TWI_SLAVE_ADDR, p_buff, len, 0)
	};
	
	err_code = app_twi_perform(&m_app_twi, twi_transfers, sizeof(twi_transfers) / sizeof(twi_transfers[0]), 0);
    APP_ERROR_CHECK(err_code);
}

static void write_reg(const uint8_t reg_addr, uint8_t* const p_buff, const uint8_t len, const uint8_t stop)
{
    ret_code_t err_code;
    uint8_t tx_buff[TWI_SLAVE_REG_LEN+1]={0};
    tx_buff[0] = reg_addr;
    memcpy(&tx_buff[1], p_buff, len);
	
	app_twi_transfer_t const twi_transfers[] = {
		APP_TWI_WRITE(TWI_SLAVE_ADDR, tx_buff, len + 1, (stop == 1) ? 0 : APP_TWI_NO_STOP)
	};
	
	err_code = app_twi_perform(&m_app_twi, twi_transfers, sizeof(twi_transfers) / sizeof(twi_transfers[0]), 0);
    APP_ERROR_CHECK(err_code);
}

static void test_case(uint8_t tx_stop)
{
	uint8_t rx_buff[TWI_SLAVE_REG_LEN] = { 0 };
    uint8_t tx_buff[] = { 0 };

    write_reg(0, tx_buff, sizeof(tx_buff), tx_stop);
    read_reg(0, rx_buff, sizeof(rx_buff));

    //NRF_LOG_RAW_HEXDUMP_DEBUG(rx_buff, sizeof(rx_buff));
}

void twi_slave_tiny_main_unwrap(void)
{
    APP_ERROR_CHECK(twi_init());
	
	//this will complete just fine
	test_case(1);
	
	//this will pull SCL low and get stuck forever
	test_case(0);
}


int main(void)
{
    // I control VDD to the I2C sensor from this pin.
    nrf_gpio_cfg_output(25);
    nrf_gpio_pin_set(25);
    nrf_delay_us(10000);

    APP_ERROR_CHECK(NRF_LOG_INIT(NULL));

    NRF_LOG_INFO("TWI master example\r\n");
    NRF_LOG_FLUSH();

    twi_slave_tiny_main_unwrap();
    
    NRF_LOG_INFO("TWI done\r\n");
    NRF_LOG_FLUSH();

    while (true)
    {
    }
}


/** @} */
